/**********************************************************************************
 * Copyright (c) 2019 Process Systems Engineering (AVT.SVT), RWTH Aachen University
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * @file manual.h
 *
 * @brief File containing the front page of the MAiNGO documentation.
 *
 **********************************************************************************/

#pragma once


/**

@mainpage McCormick-based Algorithm for mixed-integer Nonlinear Global Optimization
@authors Dominik Bongartz, Jaromił Najman, Susanne Sass, Alexander Mitsos
@date 17.02.2020

Thank you for using the beta version 0.2.1 of MAiNGO! If you have any issues, concerns, or comments, please communicate them using the <a>Issues</a> functionality at https://git.rwth-aachen.de/avt.svt/public/maingo.git or send an e-mail to MAiNGO@avt.rwth-aachen.de

<b>How to Cite MAiNGO?</b>

Please cite the latest <a href="http://permalink.avt.rwth-aachen.de/?id=729717">MAiNGO report</a>:<br>
Bongartz, D., Najman, J., Sass, S. and Mitsos, A., MAiNGO - <b>M</b>cCormick-based <b>A</b>lgorithm for mixed-<b>i</b>nteger <b>N</b>onlinear <b>G</b>lobal <b>O</b>ptimization.
Technical Report, Process Systems Engineering (AVT.SVT), RWTH Aachen University (2018).

@section intro_sec Introduction

MAiNGO can solve problems of the following form, returning a solution that is delta-feasible and epsilon-optimal (where delta and epsilon are the respective tolerances specified by the user;
 cf., e.g., M. Locatelli & F. Schoen (2013), Global Optimization: Theory, Algorithms, and Applications) or showing that no delta-feasible point exists:
\htmlonly <style>div.image img[src="NLP.PNG"]{width:6cm;align:left}</style> \endhtmlonly
\image html NLP.png

where the functions f, g and h can be computer codes implementing factorable functions (including multivariate outer functions as introduced by
Tsoukalas & Mitsos, J. Global Optim. 59 (2014) 633).
For details on what you may or may not do within these functions, see Section @ref writing_problem.
Note, however, that the relaxations and most bounding operations are not validated in the sense that round-off error is not accounted for. In this sense, the results cannot be fully guaranteed.

@subsection examples Example Applications

MAiNGO has been successfully applied to flowsheet-optimization problems (<a href="https://link.springer.com/article/10.1007/s10898-017-0547-4">Bongartz & Mitsos 2017a</a>,
 <a href="https://www.sciencedirect.com/science/article/pii/B9780444639653501070">Bongartz & Mitsos 2017b</a>,
 <a href="https://aiche.onlinelibrary.wiley.com/doi/full/10.1002/aic.16507">Bongartz & Mitsos 2019</a>), 
optimization problems with artificial neural networks (<a href="https://www.sciencedirect.com/science/article/pii/S0376738818324293">Rall et al. 2018</a>,
 <a href="https://link.springer.com/article/10.1007/s10957-018-1396-0">Schweidtmann & Mitsos 2018</a>),
and hybrid mechanistic models (<a href="https://www.sciencedirect.com/science/article/abs/pii/S009813541830886X">Schweidtmann et al. 2019</a>,
 <a href="https://www.sciencedirect.com/science/article/pii/B9780128186343501570">Schweidtmann et al. 2019</a>,
 <a href="https://www.sciencedirect.com/science/article/pii/B9780128185971500680">Huster et al. 2019a</a>,
 <a href="https://link.springer.com/article/10.1007/s11081-019-09454-1">Huster et al. 2019b</a>).

\htmlonly <style>div.image img[src="applications.png"]{width:20cm;align:left}</style> \endhtmlonly
\image html applications.png width=6cm

MAiNGO works particularly well for problems which can be formulated in a reduced space manner (<a href="https://link.springer.com/article/10.1007/s10898-017-0547-4">Bongartz & Mitsos 2017a</a>).

MAiNGO holds specialized relaxations for functions found in the field of chemical engineering (<a href="https://www.sciencedirect.com/science/article/pii/B9780444634283502721">Najman & Mitsos 2016</a>,
 <a href="https://www.sciencedirect.com/science/article/abs/pii/S0098135419309494">Najman et al. 2019</a>).
All implemented special intrinsic functions can be found at `doc/implementedFunctions/Implemented_functions.pdf`.

This manual is divided in the following sections:
- \subpage install
- \subpage writing_problem
- \subpage algorithm
- \subpage parallel_version
- \subpage special_uses
- \subpage maingo_in_your_software
- \subpage maingo_settings
- \subpage faq
- \subpage bib

\page install Installing and Executing MAiNGO

@section get_maingo Obtaining MAiNGO

In order to obtain MAiNGO, we strongly recommend using Git (Git Bash on Windows). It is possible to download MAiNGO as a .zip folder, but we strongly recommend to
use Git and follow the given instructions (in particular because we use submodules for dependencies, which need to be downloaded separately when not using Git). First you need to clone the MAiNGO repository. This is done by
\code{.sh}
git clone https://git.rwth-aachen.de/avt.svt/public/maingo.git <directory>
\endcode
where \<directory\> is the name of the directory where you can find all MAiNGO files after cloning (default name is maingo). After the cloning is done, simply navigate to the directory. In the
cloned MAiNGO directory execute
\code{.sh}
git submodule init
\endcode
If nothing happens, you are <b>not</b> in the cloned MAiNGO directory and have to navigate to there first.
For Windows users, we recommend the usage of HTTPS protocols for Git and SSH protocols for Linux and Mac OS users to avoid multiple username and password inputs. If you want to switch from HTTPS to
SSH, simply execute the shell script <tt>./switchToSsh</tt> or <tt>./switchToHttps</tt> for switchting from SSH to HTTPS and back <b>after</b> executing the <tt>git submodule init</tt> command.
For more info on SSH keys, see the Git documentation on SSH authentication which can be found under <b>Help -> SSH authentication</b> in your GitLab.
If you are having HTTPS authentication problems on Windows, please make sure that your cedential manager has the correct settings. The Windows credential manager can be found at
<b>Control Panel -> User Accounts -> Manage your credentials</b>.

In order to proceed and update all submodules of MAiNGO, execute
\code{.sh}
git submodule update -j 1
\endcode
in the MAiNGO directory. You can determine whether the above worked properly by checking if each dependency folder in the <tt>dep/</tt> folder is non-empty.
In the case that the above <b>\e DID \e NOT</b> work properly, execute the
following in the MAiNGO directory
\code{.sh}
git submodule foreach git checkout master
git submodule foreach git pull
\endcode
A note for user more familiar with Git: the <tt>git submodule update</tt> is exectued without the <tt>--recursive</tt> option. This avoids instantiating any indirect dependencies; in the repository
design of MAiNGO, all dependencies (including indirect ones) are present in the <tt>dep</tt> folder. It is also executed using only one process <tt>-j 1</tt> to avoid input failures.

@subsection update_maingo Updating MAiNGO

If you want to update to the latest MAiNGO version, use the following git commands
\code{.sh}
git pull
git submodule update
\endcode
If you changed any of the source files in MAiNGO or any dependency found in the <tt>dep/</tt> folder, you should restore the original state of MAiNGO by first saving a copy of the files you changed (or using <tt>git stash</tt>) and then executing
\code{.sh}
git checkout .
\endcode
in the MAiNGO repository and/or in any dependency repository you edited. Then, update the MAiNGO repository with the `pull` and `submodule update` commands. Finally, you can replace the files of interest by your saved copies (or retrieve them via <tt>git stash pop</tt>).


@section req_software Required Software

Building MAiNGO requires the following non-standard programs that are not in the Git repository:
- CMake 3.8 or later
- Visual Studio 2017 (Windows only)
- A Fortran Compiler (Linux and Mac OS)

CPLEX 12.8 and Knitro 11.1 are optional subsolvers that are not required for using MAiNGO, but it is strongly recommended to use CPLEX to achieve best computational results. CMake will automatically
detect whether you have CPLEX/Knitro or not. If you installed CPLEX or Knitro in a non-default path, it may be neccessary to adapt the paths in the corresponding CMakeLists.txt files at <tt>dep/knitro</tt> and <tt>dep/cplex</tt>.
A note on CPLEX versions: We experienced problems trying to include CPLEX 12.7 into MAiNGO, since there appears to be a bug in the API related to scaling of variables and equations when modifying a model causing
failure of MAiNGO for many problems. CPLEX 12.5 mostly worked for us, but did give a couple of weird bugs where problems were not updated properly. When using CPLEX 12.5, you need to go back to
Visual Studio 2010 as well (this may result in multiple changes in the CMakeLists.txt files).

All other third-party software that MAiNGO depends on comes with the MAiNGO Git. Unless you feel the need to modify the folder structure or switch to other versions (in which case we would
appreciate you contacting the MAiNGO team as well), there is nothing you need to do.


@section cmake Generating and Compiling the Project

MAiNGO uses modern target-oriented cmake commands. The <tt>CMakeLists.txt</tt> in the root directory is the sole entry point both for building MAiNGO as a standalone solver or including
it into your project.

@subsection cmake_win Windows

On Windows, only Microsoft Visual C++ 2017 is supported. We supply pre-compiled versions for all Fortran libraries, so no Fortran compiler (or runtime) should be needed.

First, start CMake and navigate or type the path to your MAiNGO directory (this is the one where the Readme.md is) and select your build directory.
\htmlonly <style>div.image img[src="CMakeHead.PNG"]{width:15cm;align:left}</style> \endhtmlonly
\image html CMakeHead.PNG width=15cm

Next, use the \e Configure button to choose Visual Studio 15 2017 Win64 as generator (or Visual Studio 15 2017 as generator and x64 as optional platform for generator in the newer versions of CMake).
Make sure that you use default native compilers. Press \e Finish and then \e Generate the project.
\htmlonly <style>div.image img[src="CMakeGenerator.PNG"]{width:25cm;align:left}</style> \endhtmlonly
\image html CMakeGenerator.PNG width=25cm

After the generation is complete, start Visual Studio 15 2017 and make sure you compile MAiNGO in \e Release mode to get the best performance.
\htmlonly <style>div.image img[src="VS_Release.PNG"]{width:6cm;align:left}</style> \endhtmlonly
\image html VS_Release.PNG width=6cm

@subsection cmake_linux_os Linux and Mac OS

On Linux and Mac OS, the following compiler configurations have been tested:
- gcc 6.3, 8.2, and 9.2 (Linux and MacOS)
- Intel C++ and Fortran Compiler 16.0.8, 17.0.7, 18.0.3, and 19.0.1 (Linux) - only works if building without the parser (MAiNGO_build_parser=false, cf. below) because of missing C++17 features in Intel compilers
- Clang 7.0, 8.0, 9.0 (Linux) and clang-1001.0.46.4 (MacOS) with gfortran - only works if not using CPLEX (CPLEX seems to be incompatible with Clang)

On Linux, the Fortran parts are usually not an issue thanks to gcc/gfortran. On Mac OS, you may need to install gfortran separately. If you use gfortran and get a linker error when compiling the code stating that a library was not found for <tt>-lgfortran</tt>, execute
\code{.sh}
export LIBRARY_PATH=\<pathToFolderContainingYourLibgfortran.a\>
\endcode
This may, for example, be <tt>/opt/local/lib/\<yourGccVersion\></tt>. You can try to locate libgfortran.a by calling <tt>locate gfortran</tt>.

We recommend to create a build folder in the MAiNGO directory first and then navigate to it (you can create the build folder anywhere else on you machine).
\code{.sh}
mkdir build
cd build
\endcode
Then simply execute cmake using the CMakeLists.txt from the MAiNGO directory by
\code{.sh}
cmake ..
\endcode
To compile the code execute
\code{.sh}
make
\endcode
You can add the option <tt>-j n</tt> to compile using n cores of your machine, e.g., execute <tt>make -j 4</tt> to compile using 4 cores.

@subsection cmake_variables CMake Variables

You can change specific CMake flags by removing or adding a checkmark at the desired option on Windows or by adding <tt>-D\<name_of_cmake_variable\></tt> after the <tt>cmake</tt> command on Linux
and Mac OS. We recommend changing only those CMake variables only that are prefixed with <tt>MAiNGO_</tt>:

- MAiNGO_build_parser (default: true): Whether to build an executable containing the parser for reading problems in ALE syntax from .txt files; uses the main file <tt>exampleAleParser/mainAleParser.cpp</tt>.
- MAiNGO_build_standalone (default: false): Whether to build an executable for solving problems via the C++ API; uses the main file <tt>exampleCppApi/mainCppApi.cpp</tt>.
- MAiNGO_use_filib (default: true): Whether to use FILIB++ for interval extensions, which is strongly recommended. If turned off, the non-validated intervals extensions from MC++ are used.
- MAiNGO_use_mpi (default: false): Whether to build the parallel version of MAiNGO. Requires an MPI library to be installed on your system.

@subsection executing_maingo Executing MAiNGO

To execute MAiNGO, open a terminal window in the build folder where the MAiNGO executable is. On Windows this is in the \e Release or \e Debug folder (depending on what you have built). In order
to not having to navigate to the build folder on Windows, you can, e.g., <b>Shift + Right click</b> on the appropriate folder and select \e Command \e Prompt or \e Power \e Shell.
If you have built the parser version of MAiNGO (i.e., with MAiNGO_build_parser=true), simply type<br>
on Windows: <tt>  MAiNGO \<yourProblem\></tt><br>
on Unix:    <tt>./MAiNGO \<yourProblem\></tt><br>

which starts MAiNGO for the problem defined in the <tt>\<yourProblem\></tt> file, e.g., <tt>MAiNGO C:/MAiNGO/exampleAleParser/problem.txt</tt>.
If you get an error message stating that the problem file has not been found, please check the path to your problem file.
If you want to modify your problem refer to section
\ref writing_problem. You can also execute MAiNGO with specific settings by calling<br>
on Windows: <tt>  MAiNGO \<yourProblem\> \<yourSettings\></tt><br>
on Unix:    <tt>./MAiNGO \<yourProblem\> \<yourSettings\></tt><br>

where <tt>\<yourSettings\></tt> describes a path to your settings file, e.g., <tt>MAiNGO C:/MAiNGO/exampleAleParser/problem.txt C:/MAiNGO/exampleAleParser/MAiNGOSettings.txt</tt>. You can find an exemplary MAiNGO settings file in <tt>exampleAleParser/MAiNGOSettings.txt</tt> or <tt>exampleCppApi/MAiNGOSettings.txt</tt>.
If you have built the standalone version in which the model is specified via the C++ API (i.e., with MAiNGO_build_standalone=true), type <br>
on Windows: <tt>  MAiNGOcpp \<yourSettings\></tt><br>
on Unix:    <tt>./MAiNGOcpp \<yourSettings\></tt><br>

\page writing_problem Modeling in MAiNGO

@section modeling_ALE Modeling with ALE

The most convenient way of modeling with MAiNGO is to use ALE (https://git.rwth-aachen.de/avt.svt/public/libale.git), which provides a framework for writing logical-algebraic expressions.
The input can be written as .txt files in ALE syntax.
An exemplary problem.txt file can be found in <tt>exampleAleParser/problem.txt</tt>.

The ALE syntax uses data types to decide which expressions can appear in which context.
All data types are constructed from the basic types **real**, **index**, and **boolean** and potentially derived types such as **set** and **tensor**.
Every expression has one of the following types:
 - **scalars** of the basic data types (e.g., **real scalar**)
 - **tensors** of the basic data types up to a maximum dimension (default: 3) (e.g., **index matrix**)
 - **sets** of scalars or tensors of the basic data types (e.g., **real vector set**)

In the context of modeling in MAiNGO, optimization variables are real scalars or tensors, while the objective function is a real scalar.
Furthermore, constraints are boolean scalars, which result from comparison operations (inequalities and equalities) on real scalars.
Finally, indexes are used to dereference tensors and sets can be used in expressions that expand over all their elements.

@subsection ALE_general File Structure

The input file is structured into sections, which are initiated with an appropriate keyword (e.g., **definitions**) followed by a **colon** (:).
Within each section, there may be zero, one, or more statements, which are each terminated by a **semicolon** (;).
At any point in the input file, the **pound symbol** (#) may be used to initiate an end-of-line comment, meaning that all input between the pound symbol and the end of the current line is ignored.
Finally, **whitespace** characters have no meaning except that they serve to separate keywords.

@subsection ALE_definitions Parameter, Variable, and Expression Symbols

Symbols are named entities that refer to fixed values, variables, or expressions of a particular type.
The names used are allowed to contain **letters**, **numbers**, and **underscores** and must start with a **letter**.
All symbols must be defined in a **definitions** section of an input file which is initiated by
\code{.sh}
definitions:
\endcode
A single input file may have arbitrarily many definitions sections that may be empty or contain arbitrarily many definitions.
However, since symbols can only be used once they have been defined, it is advisable to collect all definitions in a single definitions section at the beginning of the input file.

All definitions begin with a declarator indicating the desired data type (e.g., **real**, **index**, or **boolean**) and are terminated with a **semicolon**.

@subsubsection parameter_definitions Parameter Symbol Definitions

The following are valid definitions of different **scalar parameters** (fixed values), which are assigned using the **definition operator** (:=) operator:
\code{.sh}
real a := 12.5e-2;  # defines a scalar real parameter named "a" with value 12.5e-2
index i := 2;       # defines a scalar index parameter named "i" with value 2
boolean b := true;  # defines a scalar boolean parameter named "b" with value true
\endcode

For the definition of a **tensor parameter** its length in each dimension must be provided in square brackets after the declarator.
Furthermore, vector values are written in parentheses as a comma-separated list of entries.
Similarly, tensor values are written in terms of their subvectors (i.e., matrices are vectors of vectors, and so on).
The following are valid definitions of different **tensor parameters**:
\code{.sh}
real[3] rv := (1.3, 2.4, 3.5);     # defines a real vector parameter of length 3
index[2,2] im := ((1, 2), (3, 4)); # defines a 2x2 index matrix parameter
\endcode
For convenience, tensors can also be initialized with a scalar:
\code{.sh}
real[3,3] rm := 0; # defines a 3x3 real matrix parameter filled with zeros
\endcode

Particular entries of tensors or values of scalars can later be changed using the **assignment operator** (<-).
A **colon** (:) can be used to indicate an assignment for all values of a particular index:
\code{.sh}
i <- 1;         # assigns the value 1 to i
rm[1,2] <- 5.2; # assigns the value 5.2 to entry [1,2] of rm
rm[2,:] <- 2.2; # assigns the value 2.2 to entries [2,1], [2,2], [2,3] of rm
\endcode
In the context of assignments, it is important to note that all expressions provided for modeling are evaluated late, meaning that only the final value of a parameter symbol has any impact on the model.

For the definition of sets, the declarator is composed of the keyword **set** and the desired element type in braces.
Therein, tensor-valued elements are denoted using the same notation as above with a **colon** instead of the tensor lengths.
Similarly, set values are denoted as comma-separated lists in braces.
Contiguous index set values can be denoted in short-hand as shown below.
The following are valid definitions of different **set parameters**:
\code{.sh}
set{index} i_set := {1, 3, 5};                  # defines an index set parameter containing 1, 3, and 5
set{index} j_set := {1 .. 5};                   # defines an index set parameter containing 1 through 5
set{real[:]} v_set := {(1.3, 2.4), (3.5, 4.6)}; # defines real vector set containing two vectors
\endcode

@subsubsection variable_definitions Variable Symbol Definitions

Variable symbols can be defined using largely the same syntax as parameters with the difference that variables are not assigned values but bound intervals using the **in** operator.
Furthermore, variables can only be real scalars or tensors.
The following are valid definitions of different **scalar** and **tensor variables**:
\code{.sh}
real x;                                     # defines an unbounded real scalar variable
real[3] y in [(1, 2, 3), (2.5, 1e2, 5.4)];  # defines a bounded real vector variable
real[2,2] z in [0, ((20, 10), (23, 15))];   # defines a bounded real matrix variable with all lower bounds being zero
\endcode
Note in the above example that one or both bounds of a tensor variable may be provided as a single scalar.

Integral variables can be defined by replacing the **real** declarator by either *binary* or *integer*.
In the case of a binary variable, no bounds can be assigned ass they are implicit.
The following are valid definitions of different **binary** and **integer variables**:
\code{.sh}
binary bx;                                # defines a binary scalar variable
integer[3] iy in [(1, 2, 3), (2, 10, 5)]; # defines a bounded integer vector variable
\endcode

Similar to the assignment of parameter values, the **lower bound**, **upper bound**, and **initial point** of a variable symbol can be assigned using the **assignment operator** (<-) and the appropriate suffix for the variable name (**lb**, **ub**, or **init**):
\code{.sh}
x.lb <- -1;     # assigns the lower bound of x to be -1
x.ub <- 3;      # assigns the upper bound of x to be 3
x.init <- 2;    # assigns the initial point of x to be 2
z.lb[1,1] <- 2; # assigns the lower bound of entry [1,1] of z to be 2
\endcode

@subsubsection expression_definitions Expression Symbol Definitions

Expression symbols are similar to parameter symbols with the difference that they do not hold a basic value but rather an arbitrary expression of the appropriate type.
In contrast to parameter symbols, expression symbols can only be real, index, or boolean scalars.
The syntax for defining expression symbols is identical to the one for parameter symbols with the difference an expression is assigned instead of a constant.
For further information on what constitutes a valid expression, refer to \ref ALE_expressions.
The following are valid definitions of different **scalar expressions**:
\code{.sh}
real re := exp(x) - 4*x; # defines a real expression
index ie := 2*i + 1;     # defines an index expression
\endcode
Note that upon evaluating the input, expression symbols are substituted by their expression and don't become optimization variables.
As such, they can be used to write reduced-space formulations in a procedural way.

@subsection further_sections Further Section Headings

Beyond the definitions section, there are several sections for defining expressions such as the objective function.
While the examples provided here should be self-explanatory, a more detailed discussion of possible expressions is provided in \ref ALE_expressions.

@subsubsection objective_section Objective section

The objective function to be **minimized** must be defined in an **objective** section of an input file which is initiated by
\code{.sh}
objective:
\endcode
Each objective section may only contain a single real scalar expression with an optional description in **quotes** (" ") and terminated by a **semicolon** (;):
\code{.sh}
sin(x^2) "the objective function";
\endcode
If multiple objective sections are present in a file, the last defined objective function will be used.

@subsubsection outputs_section Outputs Ssection

Optionally, additional outputs can be defined in an **outputs** section of an input file which is initiated by
\code{.sh}
outputs:
\endcode
Each outputs section may contain arbitrarily many scalar real expressions, each with an optional description in **quotes** (" ") and terminated by a **semicolon** (;):
\code{.sh}
2 * sin(x^2) "twice the objective function";
\endcode
Upon completion of an optimization, the additional outputs will be evaluated at the optimal solution and reported.

@subsubsection constraint_section Constraints Section

Constraints must be defined in a **constraints** section of an input file which is initiated by
\code{.sh}
constraints:
\endcode
Each constraints section may contain arbitrarily many boolean scalar expressions, each with an optional description in **quotes** (" ") and terminated by a **semicolon** (;).
MAiNGO will only accept constraints that either result from a **weak inequality**, an **equality**, or a set-based expansion over a **weak inequality** or an **equality** (see \ref boolean_expressions).
Note that this restriction does not preclude the use of general boolean expressions in the context of indicator sets (see \ref set_expressions)
\code{.sh}
x <= y[1] "a simple inequality";
y[3] = 4 "a simple equality";
forall k in {1, 2} : y[k] <= y[k + 1] "a set-based inequality";
\endcode

@subsubsection relaxation_section Relaxation-only Constraints Section

Relaxation-only constraints (see \ref advanced_modeling) must be defined in a **relaxation only constraints** section of an input file which is initiated by
\code{.sh}
relaxation only constraints:
\endcode
These constraints must satisfy the same conditions as regular constraints and are denoted in the same way (see \ref constraint_section).

@subsubsection squashing_section Squashing Constraint Section

Squashing constraints (see \ref advanced_modeling) must be defined in a **squashing constraints** section of an input file which is initiated by
\code{.sh}
squashing constraints:
\endcode
These constraints must satisfy the same conditions as regular constraints and are denoted in the same way (see \ref constraint_section).
Furthermore, they must result from a **weak inequality** or a set-based expansion over a **weak inequality**.

@subsection example_input Example Input File

The following are the contents of a valid input file

\code{.sh}
definitions:
binary x;
real y in [-2,2];

real a  := 20;
real p1 := 0.2;
real p2 := 3.14159265358979323846; # ~ PI

real temp1 := -p1 * sqrt( (x^2 + y^2) / 2 ); # This is neither an optimization variable nor an equality constraint
real temp2 := (cos(p2*x) + cos(p2*y)) / 2;   # This is neither an optimization variable nor an equality constraint

x.init <- 0;
y.init <- 1;

constraints:
x <= 1;

pow(x,2) + sqr(y) = 1 "circle equality";

relaxation only constraints:
y - 1 <= 0;

y + x = 1;

outputs:
temp1 "Result of temp1";

objective: # Always minimizing
-a * exp(temp1) - exp(temp2) + a + exp(1);

\endcode

@subsection ALE_expressions Expressions

The ALE syntax for denoting expressions is similar to other established modeling languages and should be mostly intuitive.
In the following, the different expression types and the allowed operators are discussed:

@subsubsection real_expressions Scalar Real Expressions

The basic building blocks of real scalar expressions are **constants**, **symbol names**, and **entries of real tensors**.
Constants can be any real number written in floating-point or scientific notation as shown above in the definition of symbols.
Entries of tensors are accessed by bracket notation as shown above for the assignment of tensor entries.
Operators are defined for **addition** (+), **subtraction** (-), **multiplication** (*), **division** (/), and **exponentiation** (^).
Parentheses are used to enforce precedence and to denote arguments to functions.
For a complete list of implemented functions beyond the standard functions (exp(), log(), sin(), asin(), etc.), refer to doc/implementedFunctions/Implemented_functions.pdf.
The following are valid **real expressions**:
\code{.sh}
    x + sin(y[1])
    3 ^ (2 * z[1,1])
\endcode
For convenience, ALE provides the following set-based expressions that expand over a set of any given type:
\code{.sh}
    sum(k in {1 .. 3} : rv[k])
    sum(v in v_set : v[1])
    min(r in {1.2, 10.0, -1.5} : r)
    max(k in {1 .. 3} : rv[k])
\endcode
Note however, that the min and max function can also be used without set expansion:
\code{.sh}
    min(y[2], 3e-2, 5.3, x)
    max(rm[1,1], rm[2,2])
\endcode

@subsubsection index_expressions Scalar Index Expressions

Similar to real expressions, the basic building blocks of index expressions are **constants**, **symbol names**, and **entries of real tensors**.
However, index expressions and therefore also constants and the values of symbols can only take whole values.
By the same token, only the operators for **addition** (+), **subtraction** (-), and **multiplication** (*) are defined for indexes.
Parentheses are used to enforce precedence.

@subsubsection boolean_expressions Scalar Boolean Expressions

The basic building blocks of boolean expressions are **constants**, **symbol names**, and **entries of real tensors**.
Furthermore, boolean expressions result from the **comparison operators** (<, <=, =, >=, >).
Boolean expressions and therefore also constants and the values of symbols can only be **true** or **false**.
Operators are defined for *conjunction* (&), *disjunction* (|), and *negation* (!).
Parentheses are used to enforce precedence.
The following are valid **boolean expressions**:
\code{.sh}
    b & false
    x * y[1] <= 2
\endcode
Furthermore, ALE provides a set-based conjunction similar to the set-based real expressions:
\code{.sh}
    forall k in {1, 2} : y[k] <= y[k + 1]
    forall r in {1.2, 10.0, -1.5} : x >= r
\endcode

@subsubsection set_expressions Set Expressions

The basic building blocks of set expressions are **constants** and **symbol names**.
These basic sets can be refined by employing an **indicator set**, which only contains those elements that satisfy a logical condition:
\code{.sh}
    {r in {1.2, 2.3, 3.4} : r <= 3} # only contains 1.2 and 2.3
\endcode


@section modeling_cpp Modeling via C++

Another way of modeling with MAiNGO is to directly work with the C++ API. If you are not interested in the development of an own Model class simply use the exemplary <tt>C++</tt> implementation,
that can be found in <tt>exampleCppApi/problem.h</tt> and proceed with the next subsection.

For communicating an optimization problem to MAiNGO, you need to implement a specialization of the
\ref maingo.MAiNGOmodel "MAiNGOmodel" class. Your specialization of the MAiNGOmodel class needs to implement at least the following functions:
 - \ref maingo.MAiNGOmodel.get_variables "get_variables": Here you need to specify the optimization variables, which are defined by their lower and upper bounds.
   This is done by adding an appropriate \ref babBase::OptimizationVariable "OptimizationVariable" object to the variables vector.
   Optionally, each optimization variable can be given flags to determine whether they are continuous or binary (cf. \ref babBase::enums::VT "VT")
   and a branching priority which has to be at least 0. The default branching priority is 1 and a branching priority N means that MAiNGO will branch log_2(n+1) more
   often on the corresponding variable than on a variable with branching priority 1. Branching priority 0 means that MAiNGO will never branch on the variable.  <br><br>

 - \ref maingo.MAiNGOmodel.evaluate "evaluate": this function is called by MAiNGO to construt the directed acyclic graph of the objective function and constraints. The results of the evaluation
 of f, g, and h have to be written to the objective, eq, and ineq members of the \ref maingo.EvaluationContainer "EvaluationContainer" returned by the evaluate function.
 In the implementation of the evaluate function, you need to consider the following points:
        - All variables need to be of type mc::FFVar. In problem.h in the example folder, we use a typedef to call it Var instead and save some typing. Other data types (e.g., double) must
          only be used for constant parameters.

 Additionally, you may do the following in your specialization of \ref maingo.MAiNGOmodel "MAiNGOmodel":
 - Specify an initial point that is used for the first local search in multistart during pre-processing using \ref maingo.MAiNGOmodel.get_initial_point "get_initial_point".<br><br>

@subsection advanced_modeling Advanced Modeling

Within the evaluate function, you can do almost any type of computation (including calling other functions / libraries), as long as they are implemented with mc::FFVar.
It is not allowed to use loops with a number of iteration not known a priori (such as, e.g., solving a nonlinear equation system), and to use conditional statements.
For a list of intrinsic functions supported by MAiNGO, refer to the documentation in <tt>doc/implementedFunctions/Implemented_functions.pdf</tt>.

If your model does contain equations that would require iterative solution, you need to leave appropriate optimization variables and equations to the optimizer instead
(cf. Bongartz & Mitsos, J. Global Optim 69 (2017) 761).

Using conditional statements that give non-smooth or discontinuous functions is not possible directly, since computing valid relaxations always requires information on
the entire domain. Such statements can however be reformulated, either through the use of implemented non-smooth functions like the maximum of two functions, or using the
methods for relaxation of discontinuous functions introduced by Wechsung et al., J. Global Optim. 63 (2015) 1. As an alternative, one can also resort to mixed integer formulations.

For the constraints, it is important to manually scale all constraints for the relevant terms to be of the order of one, since constraint tolerances are only implemented as absolute values.

You can also specify constraints that are to be used only in the relaxation, not in the original problem (cf. Sahinidis & Tawarmalani, J. Global Optim. 32 (2005) 259).
To this end, the EvaluationContainer returned by the evaluate function contains the vectors ineqRelaxationOnly and eqRelaxationOnly. This can be useful when adding redundant constraints that would complicate the
local solution of the original problem but might tighten the relaxations. Note that adding incorrect relaxation-only constraints may lead to false infeasibility claims or other
spurious behavior of the optimization process.

You may add OutputVariables to the output vector in the EvaluationContainer. This can be used to compute and return information that is not needed during the optimization
but that you are interested in at the optimal solution.

If you are using the squash_node function, it is neccessary to introduce appropriate squash inequalities in order to maintain correctness of the optimization problem. For more information refer to
<tt>doc/implementedFunctions/Implemented_functions.pdf</tt>.

@section parsing_gams Parsing GAMS Files

We also provide a tool for parsing GAMS convert files to ALE problem.txt or MAiNGO problem.h files. For detailed description please refer to <tt>utilities/MAiNGO_Reader_Writer/</tt> and the
documentation found therein.

\page algorithm How does MAiNGO work?

MAiNGO implements a relatively basic spatial Branch-and-Bound (B&B) algorithm, enhanced with some features for range reduction and a multi-start heuristic.
The basic structure of the solver is as follows (where x includes both the integer and continuous variables):

\htmlonly <style>div.image img[src="Structure.PNG"]{width:20cm;align:left}</style> \endhtmlonly
\image html Structure.PNG width=20cm

Here, f, g, and h are bounded factorable functions (also including multivariate outer functions) that can be defined by some computer code not visible to the optimizer. The objective function
and the constraints can either be modelled in C++ or via ALE as described in \ref writing_problem. Based on this implementation,
MAiNGO will evaluate the functions to construct a directed acyclic graph represention of the model first using the MC++ library. We use automatic differentiation via FADBAD++ in order to obtain first and second derivatives of
the functions and provide them to the desired local solver. Convex and concave relaxations together with their subgradients are constructed through MC++ using the McCormick theorems, where the required intervals are obtained via FILIB++.
Note that in contrast to current state-of-the-art global solvers, MAiNGO does not introduce auxiliary variables when constructing the lower bounding problem synergizing with the reduced-space
philosophy of modelling in MAiNGO.

Each B&B node k, defined by its lower and upper bounds on the optimization variables x, is successively handed to wrappers for lower and upper bounding that communicate with the respective
subsolvers and return the respective lower (LBD) and upper (UBD) bounds, as well as the optimal solution points (see Figure above). To obtain a lower bound, a relaxation of the original problem
on the current node is constructed by relaxing the integrality requirements, replacing equalities by two inequalities each, and using the relaxations of the objective function and
constraints obtained from MC++. However, in general the McCormick relaxations are non-smooth, and therefore a further affine relaxation based on linearization using
the subgradients at one or more linearization points xLin is conducted to obtain a linear program (LP) and thus allow the use of robust LP solvers (shown for one linearization point, but
multiple points can be used in analogy):

\htmlonly <style>div.image img[src="Relaxed_LBP.PNG"]{width:12.5cm;align:left}</style> \endhtmlonly
\image html Relaxed_LBP.PNG

If MAiNGO detects the problem to be a (mixed-integer) linear program or a (mixed-integer) quadratic program, then it simply calls an available (MI)LP or (MI)QP solver to solve the problem.
Otherwise, the problem is handled as a (mixed-integer) non-linear program and the solution procedure is as follows (in the following, UBD denotes the current global upper bound on the optimal
objective value, i.e., the best objective value encountered at a feasible point so far, and LBD denotes the lower bound on the optimal objective value):

- Step 0: Preprocessing at the root node <br><br>
     - a: Initialization: Upper and lower bounding solvers and the branch-and-bound tree are initialized. The initial solution value is set to \ref maingo.Settings.infinity "infinity". <br><br>

     - b: Option check: MAiNGO checks for settings consistency, e.g., if specific heuristics are compatible with specific solvers, and informs the user about possibly modified settings.
          MAiNGO also performs a simple evaluation of McCormick relaxations at the middle point of the domain in order to directly throw interval or McCormick exceptions before even starting
          any computations (e.g., in case of division by zero). <br><br>

     - c: Constraint propagation (optional): If \ref maingo.Settings.BAB_constraintPropagation "BAB_constraintPropagation" is turned on, constraint propagation is executed for the root node.
          Since no feasible point is available yet, we set the bounds of the objective to (-infinity, infinity). If constraint propagation finds variable bounds to be inconsistent with constraints
		  this means that the problem is infeasible and MAiNGO will terminate.<br><br>

     - d: OBBT (optional): If the number of \ref maingo.Settings.PRE_obbtMaxRounds "OBBT max rounds" is >0, several rounds of optimization-based bound tightening are conducted
         (cf., e.g., Gleixner et al., J. Global Optim. 67 (2017) 731). Since up to here no feasible point is known and UBD is equal to \ref maingo.Settings.infinity "infinity", OBBT considers
         feasibility only, i.e., each variable may be maximized or minimized subject to the relaxed constraints to tighten the variable bounds. If any of the LPs solved is found to be infeasible,
         this means that the original problem was infeasible and MAiNGO will terminate. This also applies to the other instances where OBBT is used (Steps 0e and 2). The order in which variables
         are selected for OBBT is chosen according to the 'greedy' heuristic described by Gleixner et al., J. Global Optim. 67 (2017) 731.\n
         To balance the effort spent for OBBT and the gain through tightened bounds, we implemented the 'trivial filtering' heuristic described by Gleixner et al. Thus, you can adjust the
         minimum OBBT improvement (in [0,1]) to determine that a certain variable should only be considered for OBBT if this could improve
         its bound by at least \ref maingo.Settings.LBP_obbtMinImprovement "LBP_obbtMinImprovement" times the width of its interval bound in the current node. While this is not as crucial
         at the root node, it can make a big difference when using OBBT within the B&B loop. <br><br>

     - e: Multi-start (optional): If the number of \ref maingo.Settings.PRE_maxLocalSearches "maximum local searches" is >0 and steps 0c-d did not prove the problem to be infeasible, several
         runs of local optimization will be conducted using the local NLP solver specified by \ref maingo.Settings.UBP_solverPreprocessing "UBP_solverPreprocessing". If an initial point is specified by the user as part of the problem definition, it will be checked
         for feasibility first, then the first local search will start from here. The next local search starts from the midpoint of the root node, while all remaining searches start
         from randomly generated initial points within the root node.\n
         By default, output is only written about these local searches when a new incumbent (i.e., best solution point so far) is found. If you want to display the results of every local solution,
         enable \ref maingo.Settings.PRE_printEveryLocalSearch "PRE_printEveryLocalSearch". \n
         If you wish to perform a pure multi-start you can activate option \ref maingo.Settings.PRE_pureMultistart "PRE_pureMultistart". Steps 0b, 0c, 0e and 1-10 are then not performed at all
         and in step 0a only an upper bounding solver is initialized. <br><br>

     - f: Constraint propagation (optional): If \ref maingo.Settings.BAB_constraintPropagation "BAB_constraintPropagation" is turned on, constraint propagation is executed for the root node.
          If a feasible point is available, we set the bounds of the objective to (-infinity, solution value) and to (-infinity,infinity) otherwise.<br><br>

     - g: OBBT (optional): If OBBT was conducted in 0c and a feasible point was found in 0d, one additional round of OBBT is conducted that also considers optimality in additiona to feasibility.
         That is, when maximizing and minimizing each variable, a constraint that the relaxed objective function be equal to or smaller than UBD is added in addition to the
         relaxed versions of the original constraints. Locatelli & Schoen call this step 'standard range reduction'. <br><br>


- Steps 1-10: Main B&B loop (is executed while there are still nodes left in the B&B tree, or until the limits on CPU time, B&B iterations, or maximum number of nodes in memory are reached)<br><br>

    - 1: Node selection: The next node (an instance of \ref babBase::BabNode "BabNode") to be treated is selected and extracted from the B&B tree. Available heuristics for node selection
        include choosing the one with the \ref babBase::enums::NS_BESTBOUND "lowest lower bound" (default, since it guarantees convergence), as well as the \ref babBase::enums::NS_BREADTHFIRST "oldest"
        or  \ref babBase::enums::NS_DEPTHFIRST "newest" nodes in the tree. Also, the overall LBD (i.e., lowest lower bound of nodes in the B&B tree) is updated.<br><br>

    - 2: Pre-processing (constraint propagation and OBBT, optional):

        - a: If \ref maingo.Settings.BAB_constraintPropagation "BAB_constraintPropagation" is turned on, constraint propagation is executed for the current node.
             If a feasible point is available, we set the bounds of the objective to (-infinity, solution value) and to (-infinity,infinity) otherwise.<br><br>

        - b: If \ref maingo.Settings.BAB_alwaysSolveObbt "BAB_alwaysSolveObbt" is turned on, OBBT is conducted for the current node. If a feasible point has already been found,
             it considers both feasibility and optimality, otherwise it uses feasibility only. While many established global solvers (at least by default) do not perform OBBT at every node because
             the computational cost can get prohibitive for problems containing many variables (recall that we need to solve two additional LPs per variable), we decided to implement it since
             by construction the problems that MAiNGO is designed for should have relatively few variables. Here, it can pay off to set
             \ref maingo.Settings.LBP_obbtMinImprovement "LBP_obbtMinImprovement" (cf. 0c) to limit OBBT to those variables for which significant improvements can be achieved. \n
             If the node is proven to be infeasible during OBBT, it is fathomed by infeasibility and thus simply discarded (i.e., we go back directly to 10 and then back to 1).<br><br>

    - 3: Lower bounding: The lower bounding problem (LBP) is solved for the current node using one of the \ref maingo.Settings.LBP_solver "LBP solvers". If the LP is infeasible, the node is fathomed by infeasibility and thus, simply discarded (i.e., we go directly to 10 and then back to 1). Otherwise, if
         the computed lower bound is greater than the current incumbent UBD or above UBD but within tolerance, it is fathomed by value dominance and thus also discarded.<br><br>

    - 4: Upper bounding: The original problem (UBP) is solved using the local NLP solver specified by \ref maingo.Settings.UBP_solverBab "UBP_solverBab". If available, the LBP solution point is used as an initial point. Otherwise, the center of the current node is used.\n
         The solution point returned by the NLP solver is checked for
         feasibility within the user-specified feasibility tolerances (\ref maingo.Settings.deltaIneq "deltaIneq", \ref maingo.Settings.deltaEq "deltaEq") regardless of the return code of the NLP solver.
         If using binary variables, these are checked for integrality. If no NLP solver is used, a single \ref maingo::ubp::SOLVER_EVAL "function evaluation" is conducted and feasibility is
         checked at the LBP solution point or the node center. \n
         If a feasible point was found this way, it is compared to UBD and if it is better, it is stored as the new incumbent and it is represented by an asterisk '*' in the B&B output.
         The asterisk is only shown if the new incumbent is at significantly better, i.e., is not within optimality tolerances of the previous best upper bound.
         In this case, all nodes in the B&B tree (possibly including the current one) with a (node) lower bound greater than this new UBD are fathomed by value dominance and thus deleted from
         the tree.<br><br>

    - 5: Post-processing (DBBT & probing, optional): If \ref maingo.Settings.BAB_dbbt "duality-based bound tightening" (DBBT) is activated, (cf. Ryoo & Sahinidis, J. Global Optim. 8
        (1996) 107), we exploit dual information from the LBP solution to tighten bounds based on optimality. This only works if the LBP solution lies at a bound of the current node,
        and if the dual multipliers for this variable bound were successfully extracted from the underlying (MI)LP solver. \n
        If \ref maingo.Settings.BAB_probing "Probing" is enabled, the remaining bounds can be treated as well by tentatively fixing the respective variable at the bound and re-solving the LBP.
        However, experience suggests that this often too expensive (at least in the current implementation that blindly applies this to all variables). <br><br>

    - 6: Process the incumbent information: If a new incumbent has been found, the ID of the node holding the incumbent is updated. This is mainly done to enable tracking of the incumbent in
         the B\&B algorithm.<br><br>

    - 7: Branching: Two new nodes are created and inserted into the B&B tree. Options for selecting the variable to branch on include choosing the one with the \ref babBase::enums::BV "largest diameter"
         or with the \ref babBase::enums::BV_RELDIAM "largest diameter relative to the original one" (i.e., the one specified by the user). Once the branching variable has been determined,
         the point to branch at is chosen as a convex combination between the LBP solution point and the node center. Currently, we always branch in the middle of the interval (this is due to initial
         testing which gave best results when branching in the middle). If the selected branching variable is binary, the floor or ceil functions are used to push the bounds of the
         child nodes to the respective integer values.<br><br>

    - 8: Heuristic checks: Currently, it is only checked whether additional scaling in the lower bounding solver is needed. If the lower bound does not improve for
         \ref maingo.Settings.LBP_activateMoreScaling "a given number of iterations", we activate aggressive scaling in the lower bounding solver. It is planned to perform additional checks
         in this step in the future. <br><br>

    - 9: Updating statistics: All information regarding number of iterations, solved upper and lower bounding problems etc. are augmented.<br><br>

    - 10: Output: If \ref maingo.Settings.BAB_verbosity "B&B verbosity" is set to anything other than \ref maingo::VERB_NONE "VERB_NONE", output on the solution progress will be printed every
        \ref maingo.Settings.BAB_printFreq "BAB_printFreq" nodes. Additionally, output will be printed whenever a new incumbent is found (marked by an asterisk in the first column).
        Note that \ref maingo.Settings.LBP_verbosity "LBP_verbosity" and \ref maingo.Settings.UBP_verbosity "UBP_verbosity" should only be changed from their default ( \ref maingo.VERB_NONE "VERB_NONE" )
        for diagnostic reasons since they are very noisy (so is \ref maingo.VERB_ALL "VERB_ALL" for \ref maingo.Settings.BAB_verbosity "BAB_verbosity"). By default, the printed output includes:
            - the ID of the node treated
            - the lower bound of the node (after solving the LBP, i.e., not the one inherited from the parent node)
            - the overall LBD *before* treating the current node
            - the current UBD *after* treating the current node
            - the number of nodes currently in the B&B tree
            - the absolute difference between UBP and LBP
            - the relative gap between UBP and LBP
            - the CPU time spent so far <br><br>

- Step 11: Final Output
        - The incumbent as well as its objective value are returned (if a feasible point has been found) along with a message stating the termination status. Note that the returned solution
            only corresponds to the global solution if MAiNGO reports regular termination. If it terminated because of surpassed CPU time or node limits, the solution may not even be a local optimum.
            If it reports the problem to be infeasible, this means it was actually proven to be infeasible through the LBPs and/or range reduction.
        - Statistics are also given on the total number of nodes treated, the number of LBPs solved, the number of UBPs solved, and the maximum number of nodes held in memory at any point
            during solution. This values can differ since for a node that is found infeasible during OBBT, neither LBP nor UBP will be solved. If a node is found infeasible during LBP,
            UBP is omitted.
        - If additional output variables were specified as part of the problem definition, the value of these variables will be calculated at the solution point and reported at
            the very end of the solution.
        - If \ref maingo.Settings.writeLog "writeLog" is enabled, a log file is now written in the background that contains the same content as the screen output, except that the print frequency may be
            different (set through \ref maingo.Settings.BAB_logFreq "BAB_logFreq"). Currently, the log file will be written in the working directory and is named bab.log.
        - If \ref maingo.Settings.writeResFile "writeResFile" is enabled, a result file is written after a regular termination of the optimization process. The result file contains the value of all
          variables, the objective and all constraints at the final solution point. A result file is only written if the problem is feasible.
        - Finally, the overall CPU time is reported. <br><br>

@section    LBS Lower Bounding Solvers

MAiNGO currently supports four different lower bounding solvers, namely a MAiNGO internal solver, a solver based on interval arithmetics only, CLP and CPLEX. The \ref maingo::lbp::SOLVER_MAiNGO "MAiNGO" internal solver relaxes constraints
via natural interval extensions, linearizes the McCormick relaxations of the objective function at the mid point and then solves this very simple linear program. The \ref maingo::lbp::SOLVER_INTERVAL "interval-based" solver uses
interval arithmetics only in order to solve the lower bounding problem. Both solvers, \ref maingo::lbp::SOLVER_MAiNGO "MAiNGO" and  \ref maingo::lbp::SOLVER_INTERVAL "interval-based" do not support
OBBT or multiple-point linearization strategies. \ref maingo::lbp::SOLVER_CPLEX "CPLEX" is the third supported lower bounding solver and supports the full range of options implemented in MAiNGO.
Note that the current version of CPLEX cannot work with values >1e19. This is automatically detected within MAiNGO and handled appropriately. \ref maingo::lbp::SOLVER_CLP "CLP" is the fourth supported lower bounding
solver and supports the full range of options implemented in MAiNGO. Note that while CLP may be faster for small problems than CPLEX, it is more error-prone (all errors are caught by MAiNGO and handled properly).
It is recommended to use \ref maingo::lbp::SOLVER_CPLEX "CPLEX" as lower bounding solver to achieve best computational results.

@section    UBS Upper Bounding Solvers

MAiNGO currently supports local solvers found in the NLopt package, IPOPT and Knitro. Additionally, you can optimize without any local solver by the use of
\ref maingo::ubp::SOLVER_EVAL "simple function evaluations". \ref maingo::ubp::SOLVER_IPOPT "IPOPT" is an interior point optimizer. The NLopt package provides the two derivative free solvers \ref maingo::ubp::SOLVER_COBYLA "Cobyla" and \ref maingo::ubp::SOLVER_BOBYQA "Bobyqa" and two gradient-based
solvers \ref maingo::ubp::SOLVER_LBFGS "LBFGS" and \ref maingo::ubp::SOLVER_SLSQP "SLSQP". \ref maingo::ubp::SOLVER_KNITRO "Knitro" is a commercial local (MI)NLP solver consisting of many
different algorithms. For additional information, visit https://projects.coin-or.org/Ipopt , https://nlopt.readthedocs.io/en/latest/ and https://www.artelys.com/docs/knitro/2_userGuide/feasibility.html .


\page parallel_version MAiNGO - Parallel Version

There is also a parallelized version of MAiNGO. Its main use is for large problems where the B&B algorithm needs a long time to converge. The parallel version can be accessed and compiled the same way
as the sequential version. In order to generate a project of the parallel MAiNGO version please set the CMake variable <b>MAiNGO_use_mpi</b> to true and make sure you have an MPI library installed.
Depending on where it is installed, you may need to adapt the file src/FindMPI.cmake.

To run MAiNGO using MPI, type
\code{.sh}
<mpiexec> -n <numberOfProcessors> -wdir <workingDirectory> <pathToYourMAiNGOExecutable>
\endcode
where <tt>\<mpiexec\></tt> is the correct mpiexec executable of your MPI library.

It may happen that MPI asks you for an username and a password. If you type in a wrong password or a wrong username resulting in MPI not allowing you to proceed. In this case just call "mpiexec -remove" in the command window.


\page special_uses Special Uses of MAiNGO

@section maingo_multistart Using MAiNGO as a multi-start local solver:

 Simply activate the option \ref maingo::Settings::PRE_pureMultistart "PRE_pureMultistart", which disables B&B as well as range reduction at the root node, respectively.
 Set the \ref maingo::Settings::PRE_maxLocalSearches "number of max local searches" to the total number of searches you want to perform, and possibly specify an initial point, see section \ref writing_problem.
 Set your local solver using \ref maingo::Settings::UBP_solverPreprocessing "UBP_solverPreprocessing" and make sure that \ref maingo::Settings::UBP_maxStepsPreprocessing "UBP_maxStepsPreprocessing"
 is not too small. Enable \ref maingo::Settings::PRE_printEveryLocalSearch "PRE_printEveryLocalSearch" to obtain output
 from every local search. Note that in this case, the returned solution is obviously not guaranteed to be global.

@section maingo_pure_bab Using MAiNGO as a pure B&B Solver:

Set \ref maingo::Settings::PRE_obbtMaxRounds "PRE_obbtMaxRounds"=0, disable \ref maingo::Settings::BAB_dbbt "BAB_dbbt", \ref maingo::Settings::BAB_probing "BAB_probing" and
\ref maingo::Settings::BAB_alwaysSolveObbt "BAB_alwaysSolveObbt". If desired, also set \ref maingo::Settings::PRE_maxLocalSearches "PRE_maxLocalSearches"=0 to avoid multi-start at the root node.

@section maingo_feasibility_of_single_point Using MAiNGO to check feasibility of a single point:

Set up the optimization problem using arbitrary bounds containing the point of interest. Proceed like for using MAiNGO as a multi-start local solver, but
set the \ref maingo::Settings::PRE_maxLocalSearches "number of local searches" to 0.
Alternatively, you can use the C++ function  \ref maingo::MAiNGO.get_model_at_point "get_model_at_point" function to simply evaluate a point of interest.

@section maingo_epsilon_constraint Using MAiNGO to solve Bi-objective Problems:

MAiNGO implements the epsilon-constraint method for bi-objective problems, that is currently available through the C++ API only. In order to define a bi-objective problem, please take a look at
the exemplary <tt>exampleCppApi/problemEpsCon.h</tt> file. The implementation is very similar to the one described in section \ref writing_problem with the difference that now two objectives are required.
In order to solve the bi-objective problem uncomment the appropriate code lines in <tt>exampleCppApi/mainCppApi.cpp</tt>.

@section maingo_write_files_to_other_language Using MAiNGO to write Files in other Modeling Languages:

The most convenient way is to use the setting \ref maingo::Settings.writeToOtherLanguage "writeToOtherLanguage" which will automatically write a parsed file before solving your problem.
The more advanced way is to use the C++ function \ref maingo::MAiNGO::parse_maingo_to_other_language "parse_maingo_to_other_language" can be called on your MAiNGO object to automatically write
your model into a GAMS or ALE file with more options the when using the setting \ref maingo::Settings.writeToOtherLanguage "writeToOtherLanguage". An example of the usage of the function can be found in the <tt>exampleCppApi/mainCppApi.cpp</tt>. The arguments provided to the <tt>parse_maingo_to_other_language</tt> function
are a file name (default is MAiNGO_parsed_file.xxx), a solver (except if parsing to ALE; default is SCIP), whether you want to use the min and max functions or use the equivalent abs-function formulation
(default is true, meaning that the min and max functions are written to the file), whether you want to use hyperbolic functions or use equivalent formulations with the exp-function if avaiable
(default is true, meaning that the hyperbolic functions are written to the file), whether you want to ignore bounding functions which are available only within MAiNGO, see the <tt>doc/implementedFunctions/Implemented_functions.pdf</tt>
(default is true, meaning that all used bounding functions are ignored when writing files), and whether you want to write all relaxation-only constraints (default is true,
meaning that all constraints will be parsed). This function is extremely helpful if you want to do some cross-testing of solvers using the same model formulations.


\page maingo_in_your_software Using MAiNGO in Other Software

@section standalone Using MAiNGO as a Standalone Solver

If you want to use MAiNGO as a standalone solver, you can simply work with the files given in the example directory (and the CMakeLists.txt in the trunk of the Git repository) and implement
your model as described in Section \ref writing_problem. An example of a C++ model can be found in <tt>exampleCppApi/problem.h</tt>. An example for a .txt input file can be found in <tt>exampleAleParser/problem.txt</tt>.
After compiling, simply navigate to the folder where the executable was built and execute <tt>MAiNGO.exe</tt>.

The standalone version of MAiNGO can read settings for solver options from a file. An example file including descriptions and default values of all options can be found in the example folder
(<tt>MAiNGOSettings.txt</tt>). By default, MAiNGO looks for a file called <tt>MAiNGOSettings.txt</tt> in the directory where the <tt>MAiNGO.exe</tt> is located, but you can specify a custom file (including path) as a
command line argument to <tt>MAiNGO.exe</tt>. The same applies to the MAiNGO version with parser.

@section embedded Embedding MAiNGO in your Application

If you would like to embed MAiNGO in your own application, you need to link it to your software project first. Probably the easiest way it to embed MAiNGO as a submodule
of your project. For this purpose, please refer to the <tt>CMakeLists.txt</tt> in the trunk of the MAiNGO repository.

In order to use the MAiNGO object, please refer to the <tt>main.cpp</tt> in the example folder. In particular, you need to do the following things:
 - Create an instance of the \ref maingo.MAiNGO "MAiNGO" class.
 - Hand an instance of your specialization of \ref maingo.MAiNGOmodel "MAiNGOmodel" to your MAiNGO instance via its constructor or using \ref maingo.MAiNGO.set_model "set_model"
 - Specify settings using \ref maingo.MAiNGO.set_option "set_option" or \ref maingo.MAiNGO.read_settings "read_settings"
 - Solve the problem using \ref maingo.MAiNGO.solve "solve". This function will return a \ref babBase::enums::BAB_RETCODE "RETCODE".
 - Query solution information using the function provided by the \ref maingo.MAiNGO "MAiNGO" class.

@section extensions Extending MAiNGO

MAiNGO is published under the ECLIPSE license. Please refer to the <tt>LICENSE</tt> file in the MAiNGO trunk.

Regarding the implementation of MAiNGO, just explore this documentation. If you expand MAiNGO, please follow our naming conventions:
    - Variable names are written in camel case, starting with a lower case letter
    - Type names are written in camel case, starting with an upper case letter
    - Function names are all lower case, potentially with underscores
    - Enums are all capital letters
    - Private members (both functions and variables) additionally start with an underscore

\page maingo_settings MAiNGO Settings

The documentation of all settings can be found on the \ref maingo::Settings "Settings site".

When using the \ref standalone "Standalone Version" of MAiNGO, settings can be read from a file as well. The default settings file is named <tt>MAiNGOSettings.txt</tt> and contains all settings.
You can change the name of the default settings file when calling the \ref maingo.MAiNGO.read_settings "read_settings" function or simply tell maingo which settings file you want to use by
calling the MAiNGO executable with an additional argument, e.g., <tt>MAiNGO.exe MySettings.txt</tt>.


<br>
\page faq What can go wrong?

@section faq1 After starting the MAiNGO executable nothing happens

This is usually the case if some of the libraries (in most cases CPLEX) are not found. Make sure that after you installed CPLEX, the CPLEX .lib and .dll files can be found in your
environment variables. If you cannot change your environment variables, simply copy the .dll files to the <tt>MAiNGO.exe</tt> folder.<br><br>

@section faq2 MAiNGO reports a Problem to be infeasible although you expect it to be feasible

Double-check your variable bounds, model equations, and selection of constraints to make sure there are no mistakes. This is the cause in most cases.
Check whether your problem might be badly scaled and consider scaling variables and objective (note that the constraints need to be properly scaled to ensure consistent application
of tolerances, cf. Section \ref writing_problem). If the problem persists, try disabling the settings \ref maingo::Settings::BAB_alwaysSolveObbt "BAB_alwaysSolveObbt",
\ref maingo::Settings::PRE_obbtMaxRounds "PRE_obbtMaxRounds" and \ref maingo::Settings::BAB_dbbt "BAB_dbbt", which disables all range reduction. If this solves the problem,
there might be an issue in the range reduction techniques. In this case, please report the problem to the developers. Otherwise, try to perform a multi-start by setting
\ref maingo::Settings::PRE_pureMultistart "PRE_pureMultistart" to 1 and \ref maingo::Settings::PRE_maxLocalSearches "PRE_maxLocalSearches" to a high number, in addition use a powerful upper bounding solver
(e.g., \ref maingo::ubp::SOLVER_IPOPT "IPOPT") and try a local solver from multiple initial points. If a feasible point is found like this, there is an issue with the relaxations of your problem.
In this case, make sure your model does not contain conditional statements or loops with a number of iterations not known a priori. If you are using relaxation-only constraints, make sure these are actually valid.
If the problem persists, please report it to the developers. <br><br>

@section faq3 MAiNGO reports a domain violation, e.g., division by 0

This can have several causes: there may be a mistake in the model equations, the bounds on the optimization variables may be too loose, or the relaxations
(or rather the underlying interval extensions) get too bad when propagating through the model equations. In the latter case, try locating the problem within your model by, e.g., using the
mc_print(,) function.
There are two different ways to remedy such situations (except for reformulating the problem or choosing tighter bounds) depending on why the domain violation occurs:\n
If you know that the argument of interest is within the correct domain for all values of your optimization variables within their box constraints (i.e., without considering additional (linear or nonlinear) constraints),
the domain violation is purely caused by weak relaxations. In this case, you can use the functions pos(), neg(), lb_func(), ub_func() and bounding_func() to artificially cut the relaxations (cf. <tt>doc/implementedFunctions/Implemented_functions.pdf</tt>).\n
If, however, there are values of you optimization variables within their bounds (that may potentially violate other constraints) that leads to the argument actually violating the domain, the above functions must not be used.
Doing so would lead to wrong relaxations and unpredictable behavior. Instead, the points that lead to domain violations need to be excluded via a constraint and the argument can then be
restricted using the functions min(,) or max(,) (cf. <tt>doc/implementedFunctions/Implemented_functions.pdf</tt>).

@subsection faq5 MAiNGO terminates with a MC++ error saying something about alpha1 and alpha2

There seems to be a problem with the relaxations, please report this to the developers.<br><br>

@subsection faq6 MAiNGO finds feasible solutions early on, but the LBD takes forever to converge

This occurs frequently with deterministic global solvers. It is usually a sign for the relaxations being weak. Sometimes it helps to start with tighter bounds on the optimization variables. Otherwise, more effort might
be needed to find better relaxations. If not doing so already, you should also consider using range reduction (cf. Section @ref algorithm).<br><br>

@subsection faq7 MAiNGO runs forever without finding a feasible point although LBD seems to be converged already

Try using a more powerful UBP solver (e.g., IPOPT or KNITRO), possibly increase \ref maingo::Settings::UBP_maxStepsBab "UBP_maxStepsPreprocessing". Also set \ref maingo::Settings::PRE_maxLocalSearches "PRE_maxLocalSearches"> to 0 in order to
enable local searches at the root node. If the problem persists, it might be problematic for local solvers. Try a more powerful solver which may currently not be implemented in MAiNGO and
use the point reported by the external solver as an initial point for MAiNGO. Previous experience has shown that one possible cause can be redundant constraints (since these destroy
constraint qualifications). If it seems like the lower bound converged, try using simple \ref maingo::ubp::SOLVER_EVAL "function evaluations". If your problem contains binary variables, improvements may be needed to the upper bounding strategy that currently does not distinguish between continuous
and binary variables except that the latter are checked for integrality at the returned solution point. In this case, please report the problem to the developers.<br><br>

\page maingo_settings MAiNGO Settings

\page bib Where can I read more?

Some more general information on MAiNGO can be found in the latest <a href="http://permalink.avt.rwth-aachen.de/?id=729717">MAiNGO report</a>:<br>
    - D. Bongartz, J. Najman, S. Sass, and A. Mitsos, MAiNGO - <b>M</b>cCormick-based <b>A</b>lgorithm for mixed-<b>i</b>nteger <b>N</b>onlinear <b>G</b>lobal <b>O</b>ptimization. Technical Report, Process Systems Engineering (AVT.SVT), RWTH Aachen University (2018).
    
@section readRelaxations Relaxations
The relaxations implemented in MC++ that are used in MAiNGO were introduced in the following works (details on the implementation can be found in the documentation of MC++):
    - G.P. McCormick, Computability of global solutions to factorable nonconvex programs: Part I - Convex underestimating problem, Mathematical Programming 10 (1976) 145-175.
    - A. Mitsos, B. Chachuat, and P.I. Barton, McCormick-Based Relaxations of Algorithms, SIAM Journal on Optimization 20 (2009) 573-601.
    - J.K. Scott, M.D. Stuber, and P.I. Barton, Generalized McCormick Relaxations, Journal of Global Optimization  51 (2011) 569-606.
    - A. Tsoukalas and A. Mitsos, Multivariate McCormick Relaxations, Journal of Global Optimization 59 (2014) 633-662.
    - J. Najman, D. Bongartz, A. Tsoukalas, and A. Mitsos, Erratum to: Multivariate McCormick Relaxations, Journal of Global Optimization  68 (2017) 219-225.
    - A. Wechsung and P.I. Barton, Global optimization of bounded factorable functions with discontinuities, Journal of Global Optimization 58 (2014) 1-30.

A discussion of their convergence properties can be found in:
    - A. Bompadre and A. Mitsos, Convergence rate of McCormick relaxations, Journal of Global Optimization 52 (2012) 1-28.
    - J. Najman and A. Mitsos, Convergence analysis of multivariate McCormick relaxations, Journal of Global Optimization 66 (2016) 597-628.
	- J. Najman and A. Mitsos, On tightness and anchoring of McCormick and other relaxations, Journal of Global Optimization 74 (2019) 677-703.

Examples for the construction of tighter relaxations for special functions can be found in:
    - J. Najman and A. Mitsos, Convergence Order of McCormick Relaxations for LMTD function in Heat Exchanger Networks, In: 26th European Symposium on Computer Aided Process Engineering (2016) 1605-1610.
    - A. Schweidtmann and A. Mitsos, Global Deterministic Optimization with Artificial Neural Networks Embedded, Journal of Optimization Theory and Applications 180(3) (2019).
    - J. Najman, D. Bongartz, and A. Mitsos, Relaxations of thermodynamic property and costing models in process engineering. Computers & Chemical Engineering 130 (2019) 106571.
    - J. Najman, D. Bongartz, and A. Mitsos, Convex Relaxations of Componentwise Convex Functions. Computers & Chemical Engineering 130 (2019) 106527.

Further extensions that are currently not implemented include the following:
    - M.D. Stuber, J.K. Scott, and P.I. Barton, Convex and concave relaxations of implicit functions, Optimization Methods and Software 30 (2015) 424-460.
    - A. Wechsung, J.K. Scott, H.A. Watson, and P.I. Barton, Reverse propagation of McCormick relaxations, Journal of Global Optimization 63 (2015) 1-36.
    - K.A. Khan, H.A. Watson, and P.I. Barton, Differentiable McCormick Relaxations, Journal of Global Optimization 67 (2017) 687-729.

@section readBaB Branch-and-Bound
Branch-and-Bound for continuous optimization is attributed to:
    - J.E. Falk and R.M. Soland, An algorithm for separable nonconvex programming problems, Management Science 15 (1969) 550-569.

Basic results about convergence properties as well as a discussion of node selection, branching schemes etc can be found in:
    - R. Horst and H. Tuy, Global Optimization: Deterministic approaches, Springer Science & Business Media, Berlin (1996).
    - M. Locatelli and F. Schoen, Global Optimization: Theory, algorithms, and applications, MIS-SIAM, Philadelphia (2013).
    - M. Tawarmalani and N.V. Sahinidis, Convexification and Global Optimization in Continuous and Mixed-Integer Nonlinear Programming, Kluwer Academic Publishers (2002).

The structure of MAiNGO's algorithm (in particular the iterative application of node pre-processing, LBP, UBP, and node post-processing) is analogous to the one described in the latter.

@section readRR Range Reduction
An overview of range reduction techniques can be found in:
    - Y. Puranik and N.V. Sahinidis, Domain reduction techniques for global NLP and MINLP optimization, Constraints (2017), doi:10.1007/s10601-016-9267-5.
    - M. Locatelli and F. Schoen, Global Optimization: Theory, algorithmy, and applications, MIS-SIAM, Philadelphia (2013).

Duality-based bound tightening (DBBT) as well as probing are taken from:
    - H.S. Ryoo and N.V. Sahinidis, Global optimization of nonconvex NLPs and MINLPs with applications in process design, Computers & Chemical Engineering 19 (1995) 551-566.
    - H.S. Ryoo and N.V. Sahinidis, A branch-and-reduce approach to global optimization, Journal of Global Optimization 8 (1996) 107-138.

A detailed discussion of optimization-based bound tightening (OBBT) as well as the trivial filtering and greedy heuristic implemented in MAiNGO can be found in:
    - A.M. Gleixner, T. Berthold, B. Müller, and S. Weltge, Three enhancements for optimization-based bound tightening, Journal of Global Optimization 67 (2017) 731-757.

McCormick subgradient-based interval heuristic:
    - J. Najman, A. Mitsos, Tighter McCormick relaxations through subgradient propagation. Journal of Global Optimization (2019).

@section readUses Uses of MAiNGO
Examples of applications of earlier versions of MAiNGO can be found in:
    - D. Bongartz and A. Mitsos, Deterministic Global Optimization of Process Flowsheets in a Reduced Space Using McCormick Relaxations, Journal of Global Optimization 69 (2017) 761-796.
    - D. Bongartz and A. Mitsos, Infeasible Path Global Flowsheet Optimization Using McCormick Relaxations, In: Proceedings of the 27th Symposium on Computer Aided Process Engineering - ESCAPE27 (2017) 631-636.
    - D. Bongartz and A. Mitsos, Deterministic Global Flowsheet Optimization: Between Equation-Oriented and Sequential-Modular Methods, AIChE Journal 65 (2019) 1022-1034.
	- W.R. Huster, D. Bongartz, and A. Mitsos, Deterministic Global Optimization of the Design of a Geothermal Organic Rankine Cycle, Energy Procedia 129 (2017) 50-57.
	- W.R. Huster, A.M. Schweidtmann and A. Mitsos, Impact of accurate working fluid properties on the globally optimal design of an organic Rankine cycle, Computer Aided Chemical Engineering 47 (2019) 427-432.
	- W.R. Huster, A.M. Schweidtmann and A. Mitsos, Working fluid selection for organic rankine cycles via deterministic global optimization of design and operation, Optimization and Engineering (2019) in press.
    - D. Rall, D. Menne, A.M. Schweidtmann, J. Kamp, L. von Kolzenberg, A. Mitsos and Matthias Wessling, Rational design of ion separation membranes, Journal of Membrane Science 569 (2019) 209-219.
	- P. Schäfer, A.M. Schweidtmann, P.H.A. Lenz, H.M.C. Markgraf, A. Mitsos, Wavelet-based grid-adaptation for nonlinear scheduling subject to time-variable electricity prices, Computers and Chemical Engineering (2019) in press.
	- A.M. Schweidtmann, D. Bongartz, W.R. Huster, A. Mitsos, Deterministic Global Process Optimization: Flash Calculations via Artificial Neural Networks, Computer Aided Chemical Engineering 46 (2019) 937-942.
	- A.M. Schweidtmann and A. Mitsos, Deterministic Global Optimization with Artificial Neural Networks Embedded, Journal of Optimization Theory and Applications 180 (2019) 925-948.
	- A.M. Schweidtmann, W.R. Huster, J.T. Lüthje and A. Mitsos, Deterministic global process optimization: Accurate (single-species) properties via artificial neural networks, Computers & Chemical Engineering 121 (2019) 67-74.
*/